/**
 * 支持以下坐标系：
 * 1. 经纬度
 * 2. 偏移经纬度，用于解决高精度抖动问题
 * 3. 瓦片坐标，用于数据瓦片
 * 4. 常规世界坐标系，用于常规 2D/3D 可视化场景
 * 5. P20
 * @see https://yuque.antfin-inc.com/yuqi.pyq/fgetpa/doml91
 */
export declare enum CoordinateSystem {
    LNGLAT = 1,
    LNGLAT_OFFSET = 2,
    VECTOR_TILE = 3,
    IDENTITY = 4,
    P20 = 5,
    P20_OFFSET = 6,
    METER_OFFSET = 7,
    P20_2 = 8
}
export declare const CoordinateUniform: {
    CoordinateSystem: string;
    ViewportCenter: string;
    ViewportCenterProjection: string;
    PixelsPerDegree: string;
    PixelsPerDegree2: string;
    PixelsPerMeter: string;
    Mvp: string;
};
export interface ICoordinateSystemService {
    needRefresh: boolean;
    refresh(offset?: [number, number]): void;
    getCoordinateSystem(): CoordinateSystem;
    setCoordinateSystem(coordinateSystem: CoordinateSystem): void;
    getViewportCenter(): [number, number];
    getViewportCenterProjection(): [number, number, number, number];
    getPixelsPerDegree(): [number, number, number];
    getPixelsPerDegree2(): [number, number, number];
    getPixelsPerMeter(): [number, number, number];
}
